/*
 * ============================================================================
 * DemoPersonFilter.java: Exercise filter capabilities ...
 * ============================================================================
 */

package filter02;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

public class DemoPersonFilter {
   public static void main(String[] args) {

      System.out.printf("DemoPersonFilter ...\n");
      System.out.printf("====================================== ...\n");

      // Assemble and print array list of person objects ...

      System.out.printf("--- Assemble array list of person objects ...\n");

      List<Person> persons = Arrays.asList(
          new Person(    "Mark", 30),
          new Person(    "Nina", 12),
          new Person( "Leonard", 12),
          new Person(   "Maria", 12),
          new Person(   "Maria", 20),
          new Person(   "Clara", 25),
          new Person(  "Angela", 21)
      );

      System.out.println(persons);

      // Retrieve and print person by name (Java 7 and before).

      System.out.printf("--- Part 1: Retrieve and print person by name (Java 7) ...\n");

      Person result = getStudentByName(persons, "Mark");
      System.out.println(result);

      // Create data stream and filter ...

      System.out.printf("--- Part 2: Create data stream and filter ...\n");
      System.out.printf("--- Find: Maria ...\n");

      Person result1 = persons.stream()                      // Convert to steam
                .filter(x -> "Maria".equals(x.getName()))     // we want "Maria" only
                .findAny()                                   // If 'findAny' then return found
                .orElse(null);                               // If not found, return null

      System.out.printf("--- Result: %s ...\n", result1.toString() );

      System.out.printf("--- Find: Leonard ...\n");
        
      Person result2 = persons.stream()
                .filter(x -> "Leonard".equals(x.getName()))
                .findAny()
                .orElse(null);

      System.out.printf("--- Result: %s ...\n", result2.toString() );

      System.out.printf("--- Part 3: Find Maria who has age 20 (style 1) ...\n");

      Person result3 = persons.stream()
              .filter((p) -> "Maria".equals(p.getName()) && 20 == p.getAge())
              .findAny()
              .orElse(null);

      System.out.printf("--- Result: %s ...\n", result3.toString() );

      System.out.printf("--- Part 4: Find Maria who has age 20 (style 2) ...\n");

      Person result4 = persons.stream()
                .filter(p -> {
                    if ("Maria".equals(p.getName()) && 20 == p.getAge()) {
                        return true;
                    }
                    return false;
                }).findAny()
                .orElse(null);

      System.out.printf("--- Result: %s ...\n", result4.toString() );

      System.out.printf("--- Part 5: Demo filter and map ...\n");

      String name = persons.stream()
                .filter(x -> "Nina".equals(x.getName()))
                .map(Person::getName)                        //convert stream to String
                .findAny()
                .orElse("");

      System.out.println("name : " + name);

      System.out.printf("--- Part 6: Create list of person names ...\n");

      List<String> collect = persons.stream().map(Person::getName).collect(Collectors.toList());
      collect.forEach(System.out::println);

      System.out.printf("--- Part 7: Create list of person ages ...\n");

      List<Integer> collect02 = persons.stream().map(Person::getAge).collect(Collectors.toList());
      collect02.forEach(System.out::println);

      System.out.printf("====================================== ...\n");
      System.out.printf("Finished ...\n");
   }

   // ================================================
   // Retrieve student by name (Java 7 and before) ...
   // ================================================

   private static Person getStudentByName(List<Person> persons, String name) {
      Person result = null;
      for (Person temp : persons) {
          if (name.equals(temp.getName())) {
              result = temp;
          }
      }
      return result;
   }
}
