/*
 *  ==================================================================
 *  DeepCopyTest.java: Illustrate the difference between shallow copy
 *  and a deep copy of a HashMap.
 * 
 *  Written by: Mark Austin                                March, 2013 
 *  ==================================================================
 */

package hashmap04;

import java.util.HashMap;
import java.util.Iterator;

public class DeepCopyTest {
   public static void main( final String[] args ) {
      testReference();
      testDeepCopy();
   }

   // Deep Copy Test ....
 
   private static void testDeepCopy() {

      System.out.println( "" );
      System.out.println( "Deep Copy Test" );
      System.out.println( "========================" );

      HashMap<Integer, String> data1 = new HashMap<Integer, String>();
      data1.put( new Integer( "1" ), "thing one" );
      data1.put( new Integer( "2" ), "thing two" );
      data1.put( new Integer( "3" ), "thing three" );

      HashMap<Integer, String> data2 = new HashMap<Integer, String>( data1 );
      printHashMap( data1, "data1" );
      printHashMap( data2, "data2" );

      System.out.println( "Remove data in data2" );

      data1.remove( new Integer( 2 ) );
      data1.put( new Integer( "2" ), "thing two (version2)" );

      printHashMap( data1, "data1" );
      printHashMap( data2, "data2" );

      System.out.println( "========================" );
   }

   // A HashMap is referenced by two variables, data1 and data2....

   private static void testReference() {

      System.out.println( "" );
      System.out.println( "Simple Copy Test" );
      System.out.println( "========================" );

      HashMap<Integer, String> data1 = new HashMap<Integer, String>();
      data1.put( new Integer( "1" ), "one" );
      data1.put( new Integer( "2" ), "two" );
 
      HashMap<Integer, String> data2 = data1;
      printHashMap( data1, "data1" );
      printHashMap( data2, "data2" );
 
      System.out.println( "Remove data in data2" );

      data1.remove( new Integer( 2 ) );
      printHashMap( data1, "data1" );
      printHashMap( data2, "data2" );

      System.out.println( "========================" );
   }
 
   private static void printHashMap( HashMap<Integer, String> map, String message ) {
      System.out.println( "---- " + message + " Begin ----" );

      final Iterator iterator = map.keySet().iterator();
      while ( iterator.hasNext() ) {
         final Integer key = (Integer)iterator.next();
         final String value = map.get( key );
 
         System.out.println( key + " " + value );
      }

      System.out.println( "---- " + message + " End ----" );
   }
}

