/**
 * ==================================================================
 * SymbolTableImpl.java: An implementation of the symbol table.
 *
 * Copyright (c) 2009 by Ronald Mak
 * For instructional purposes only.  No warranties.
 * ==================================================================
 */

package symbol02;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.TreeMap;

import symbol02.*;

public class SymbolTableImpl extends TreeMap<String, SymbolTableEntry> implements SymbolTable {
    private int nestingLevel;

    public SymbolTableImpl(int nestingLevel) {
        this.nestingLevel = nestingLevel;
    }

    // Get nesting level ....

    public int getNestingLevel() {
        return nestingLevel;
    }

    /**
     * Create and enter a new entry into the symbol table.
     */

    public SymbolTableEntry enter(String name) {
        SymbolTableEntry entry = SymbolTableFactory.createSymbolTableEntry(name, this);
        put(name, entry);
        return entry;
    }

    /**
     * Look up an existing symbol table entry.
     */

    public SymbolTableEntry lookup(String name) {
        return get(name);
    }

    // Return a list of symbol table entries sorted by name.

    public ArrayList<SymbolTableEntry> sortedEntries() {
        Collection<SymbolTableEntry> entries = values();
        Iterator<SymbolTableEntry>  iter = entries.iterator();
        ArrayList<SymbolTableEntry> list = new ArrayList<SymbolTableEntry>(size());

        // Iterate over the sorted entries and append them to the list.

        while (iter.hasNext()) {
            list.add(iter.next());
        }

        return list;  // sorted list of entries
    }
}
