/**
 * ===================================================================================
 * TestSymbolTable.java: Exercise symbol table features ...
 *
 * Written by: Mark Austin                                                   June 2013
 * ===================================================================================
 */

package symbol02;

import java.util.HashMap;

import symbol02.datatypes.Dimensions;
import symbol02.datatypes.DimensionsLibrary;
import symbol02.datatypes.Quantity;
import symbol02.datatypes.Function;
import symbol02.datatypes.Max;
import symbol02.datatypes.Min;

public class TestSymbolTable {
    public static void main ( String args[] ) {

       System.out.printf("\n");
       System.out.printf("--- Exercise SymbolTableFactory ... \n");
       System.out.printf("--- =========================================== \n");
       System.out.printf("\n");

       SymbolTableFactory stf = new SymbolTableFactory();
       SymbolTableStack   sts = stf.createSymbolTableStack();

       System.out.printf("--- Symbol Table: Level 0 ... \n");
       System.out.printf("--- Add strings to Symbol Table ... \n");

       SymbolTable st00  = stf.createSymbolTable(0);
       st00.enter("pi");
       st00.enter("b string");
       st00.enter("c string");
       st00.enter("d string");

       String targetName = "pi";
       SymbolTableEntry target01 = st00.lookup(targetName);
       target01.setAttribute( SymbolTableKeyImpl.CONSTANT, new Double(3.141) );
       targetName = "b string";
       SymbolTableEntry target02 = st00.lookup(targetName);
       target02.setAttribute( SymbolTableKeyImpl.VAR, new Integer(2) );
       targetName = "c string";
       SymbolTableEntry target03 = st00.lookup(targetName);
       target03.setAttribute( SymbolTableKeyImpl.DATA, new Integer(2) );
       targetName = "d string";
       SymbolTableEntry target04 = st00.lookup(targetName);
       target04.setAttribute( SymbolTableKeyImpl.DATA, new Integer(8) );

       System.out.println ( "--- Target01: " + target01.toString() );
       System.out.println ( "---   Name01: " + target01.getName()  );
       System.out.println ( "--- Target02: " + target02.toString() );
       System.out.println ( "---   Name02: " + target02.getName()  );
       System.out.println ( "--- Target03: " + target03.toString() );
       System.out.println ( "---   Name03: " + target03.getName()  );
       System.out.println ( "--- Target04: " + target04.toString() );
       System.out.println ( "---   Name04: " + target04.getName()  );

       System.out.printf("--- Symbol Table: Level 1 ... \n");

       SymbolTable st01  = stf.createSymbolTable(1);
       st01.enter("0 string");
       st01.enter("1 string");
       st01.enter("5 string");
       st01.enter("2 string");
       st01.enter("6 string");

       System.out.printf("--- Symbol Table: Level 2 ... \n");

       SymbolTable st02  = stf.createSymbolTable(2);
       st02.enter("xyz string");

       sts.push ( st00 );
       sts.push ( st01 );
       sts.push ( st02 );

       targetName = "b string";
       SymbolTableEntry targetId = sts.lookup(targetName);
       if (targetId == null) {
           System.out.println("--- targetID not found! ");
           targetId = sts.enterLocal(targetName);
       } 

       // ===============================================
       // Add Dimension objects to the symbol table ...
       // ===============================================

       System.out.printf("\n");
       System.out.printf("--- Add Dimension objects to the symbol table ...\n");
       System.out.printf("--- =============================================\n");
       System.out.printf("\n");

       DimensionsLibrary lib = new DimensionsLibrary();
       HashMap library = lib.getLibrary();

       Dimensions d01 = (Dimensions) library.get("m");
       Dimensions d02 = (Dimensions) library.get("mm");
       Dimensions d03 = (Dimensions) library.get("kPa");

       st01.enter( d01.getName() );
       st01.lookup( d01.getName() ).setAttribute( SymbolTableKeyImpl.DIMENSION, d01 );
       st01.enter( d02.getName() );
       st01.lookup( d02.getName() ).setAttribute( SymbolTableKeyImpl.DIMENSION, d02 );
       st01.enter( d03.getName() );
       st01.lookup( d03.getName() ).setAttribute( SymbolTableKeyImpl.DIMENSION, d03 );

       // ===============================================
       // Add Quantity objects to the symbol table ...
       // ===============================================

       System.out.printf("\n");
       System.out.printf("--- Add Quantity objects to the symbol table ...  \n");
       System.out.printf("--- ============================================= \n");
       System.out.printf("\n");

       Quantity q01 = new Quantity( "a1" );
       q01.setValue ( 2.3 );
       q01.setDimensions( d03 );

       st02.enter(  q01.getName() );
       st02.lookup( q01.getName() ).setAttribute( SymbolTableKeyImpl.VAR, q01 );

       // ===============================================
       // Add builtin functions to the symbol table ...
       // ===============================================

       System.out.printf("\n");
       System.out.printf("--- Add builtin functions to the symbol table ... \n");
       System.out.printf("--- ============================================= \n");
       System.out.printf("\n");

       // Create references to builtin functions ...

       Function m = new Max();
       Function n = new Min();

       // Add builtin functions to symbol table ...

       st02.enter(  m.getName() );
       st02.lookup( m.getName() ).setAttribute( SymbolTableKeyImpl.BUILTIN, m );
       st02.enter(  n.getName() );
       st02.lookup( n.getName() ).setAttribute( SymbolTableKeyImpl.BUILTIN, n );

       // ===============================================
       // Print contents of symbol table ...
       // ===============================================

       System.out.printf("\n");
       System.out.printf("--- Print contents of symbol table stack ... \n");
       System.out.printf("--- ============================================= \n");
       System.out.printf("\n");

       System.out.printf("--- Print st00 ... \n");
       System.out.printf("\n");

       System.out.println ( st00.sortedEntries().toString() );

       System.out.printf("\n");
       System.out.printf("--- Print st01 ... \n");
       System.out.printf("\n");

       System.out.println ( st01.sortedEntries().toString() );

       System.out.printf("\n");
       System.out.printf("--- Print st02 ... \n");
       System.out.printf("\n");

       System.out.println ( st02.sortedEntries().toString() );

       System.out.printf("\n");
       System.out.printf("--- Query symbol table stack, interrogate results ... \n");
       System.out.printf("--- ================================================= \n");
       System.out.printf("\n");

       System.out.printf("--- Lookup constant \"pi\" ... \n");

       SymbolTableEntry entry01 = sts.lookup("pi");
       if (entry01 == null) {
          System.out.println("--- Search: pi not found! ");
       } else {
          System.out.println("--- Search: pi found! ");
          SymbolTable s = entry01.getSymbolTable();
          System.out.println("---         nesting level = " + s.getNestingLevel() );
          Object obj = entry01.getAttribute( SymbolTableKeyImpl.CONSTANT );
          if ( obj instanceof Double ) {
               Double v = (Double) obj;
               System.out.println("---         value = " + v.toString() );
          }
       }

       System.out.printf("--- Lookup variable \"a1\" ... \n");

       SymbolTableEntry entry02 = sts.lookup("a1");
       if (entry02 == null) {
          System.out.println("--- Search: a1 not found! ");
       } else {
          System.out.println("--- Search: a1 found! ");
          SymbolTable s = entry02.getSymbolTable();
          System.out.println("---        nesting level = " + s.getNestingLevel() );

          Object obj = entry02.getAttribute( SymbolTableKeyImpl.VAR );
          if ( obj instanceof Quantity ) {
               Quantity q = (Quantity) obj;
               System.out.println("---         details = " + q.toString() );
          }
       } 

       // =========================================================
       // Retrieve and execute builtin functions ...
       // =========================================================

       System.out.printf("\n");
       System.out.printf("--- Part 01: Compute Max(1,5) .... \n");
       System.out.printf("\n");

       SymbolTableEntry entry03 = sts.lookup("Max");
       if (entry03 == null) {
          System.out.println("--- Search: Builtin function Max() not found! ");
       } else {
          System.out.println("--- Search: Builtin function Max() found! ");
          SymbolTable s = entry03.getSymbolTable();
          System.out.println("---         nesting level = " + s.getNestingLevel() );
          Object obj = entry03.getAttribute( SymbolTableKeyImpl.BUILTIN );
          if ( obj instanceof Function ) {
             Function k = (Function) obj;
             k.setValues ( 1.0, 5.0 );
             double result = k.compute();
             System.out.println("--- Function name = " + k.getName() );
             System.out.println("--- No args    = " + k.getNoArgs() );
             System.out.println("--- Max result = " + result );
          }
       }

       System.out.printf("\n");
       System.out.printf("--- Part 02: Compute Max(1,2,3,5) ... \n");
       System.out.printf("\n");

       SymbolTableEntry entry04 = sts.lookup("Max");
       if (entry04 == null) {
          System.out.println("--- Search: Builtin function Max() not found! ");
       } else {
          System.out.println("--- Search: Builtin function Max() found! ");
          SymbolTable s = entry04.getSymbolTable();
          System.out.println("---          nesting level = " + s.getNestingLevel() );
          Object obj = entry04.getAttribute( SymbolTableKeyImpl.BUILTIN );
          if ( obj instanceof Function ) {
             Function k = (Function) obj;
             k.setValues ( 1.0, 2.0, 3.0, 5.0 );
             double result = k.compute();
             System.out.println("--- Function name = " + k.getName() );
             System.out.println("--- No args    = " + k.getNoArgs() );
             System.out.println("--- Max result = " + result );
          }
       }

       System.out.printf("\n");
       System.out.printf("--- Part 03: Compute Min( 1, 2,-3, 5) .... \n");
       System.out.printf("\n");

       SymbolTableEntry entry05 = sts.lookup("Min");
       if (entry05 == null) {
          System.out.println("--- Search: Builtin function Min() not found! ");
       } else {
          System.out.println("--- Search: Builtin function Min() found! ");
          SymbolTable s = entry05.getSymbolTable();
          System.out.println("---         nesting level = " + s.getNestingLevel() );
          Object obj = entry05.getAttribute( SymbolTableKeyImpl.BUILTIN );
          if ( obj instanceof Function ) {
             Function k = (Function) obj;
             k.setValues ( 1.0, 2.0, -3.0, 5.0 );
             double result = k.compute();
             System.out.println("--- Function name = " + k.getName() );
             System.out.println("--- No args    = " + k.getNoArgs() );
             System.out.println("--- Min result = " + result );
          }
       }

       System.out.printf("--- ============================================ ... \n");
       System.out.printf("--- Finished ... \n");
    }
}
